<?php
/**
 * Class for handle ajax request
 */
if ( ! class_exists( 'IPProf_Ajax' )) {

    class IPProf_Ajax
    {
        /**
         * Take ajax action name
         *
         * @var string
         */
        protected $_action;
        
        /**
         * Take ajax action handlers
         *
         * @var array
         */
        protected $_handlers = array(
            'ipprof_update'            => 'update',
            'ipprof_delete'            => 'delete',
            'ipprof_offer_update'      => 'offer_update',
            'ipprof_offer_delete'      => 'offer_delete',
            'ipprof_offer_sort'        => 'offer_sort',
            'ipprof_offer_get_edit'    => 'offer_get_edit',
            'ipprof_get_slug'          => 'get_slug',
            'ipprof_setting'           => 'setting',
            'ipprof_savestat'          => 'save_stat',
            'ipprof_getchart'          => 'get_chart',
            'ipprof_getmore'           => 'get_more'
        );
        
        /**
         * Class constructor
         *
         * @return void
         */
        public function __construct()
        {
        }
        
        /**
         * Get action name
         *
         * @return string
         */
        public function getAction()
        {
            return $this->_action;
        }
        
        /**
         * Set action
         *
         * @param  string $action
         * @return IPShortcode_Ajax
         */
        public function setAction( $action )
        {
            $this->_action = sanitize_title( $action );
        }
        
        /**
         * Execute request
         *
         * @param  string $action
         * @return IPShortcode_Ajax
         */
        public function exec()
        {
            $data = array_merge( $_GET, $_POST );
            if ( empty( $data['doajax'] ) || empty( $data['ipprof'] ) || empty( $data['action'] ) ) {
                return true;
            }
            $this->setAction( $data['action'] );
            
            $action   = $this->getAction();
            $handlers = (array)$this->_handlers;
            if ( array_key_exists( $action, $handlers )) {
                $handler = $handlers[$action];
                if ( method_exists( $this, $handler )) {
                    $this->$handler();
                }
            }
            return true;
        }
        
        /**
         * Update
         *
         * @return void
         */
        public function update()
        {
            $redirect = false;
            $is_add   = false;
            $error    = true;
            if ( ! empty( $_POST['isadd'] )) {
                $is_add = true;
            }
            if ( ! empty( $_POST['profit_id'] )) {
                $profitId = absint( $_POST['profit_id'] );
                if ( $profit = get_post( $profitId ) ) {
                    $this->_update_profit( $profit );
                } else {
                    $profitId = 0;
                }
            } else {
                $profitId = $this->_add_profit();
            }
            if ( $profitId )
            {
                $error = false;
                $model = new IPProf_Model();
                $model->save_setting( $profitId );
            }
            if ( $error ) {
                $status  = 0;
                $profitId  = 0;
                if ( $is_add ) {
                    $message = __( 'Failed to create a profit page.', 'wp_profit_page_creator' );
                } else {
                    $message = __( 'Failed to save profit page.', 'wp_profit_page_creator' );
                }
            } else {
                $status = 1;
                if ( $is_add ) {
                    $message  = __( 'Successfully create a profit page.', 'wp_profit_page_creator' );
                    $redirect = admin_url() . 'admin.php?page=ipprof&action=edit&id=' . $profitId . '&create=1&tab=ipprof-section-offer';
                } else {
                    $message = __( 'Successfully to save profit page.', 'wp_profit_page_creator' );
                }
            }
            $result = array(
                'status'   => $status,
                'message'  => $message,
                'profitid'  => $profitId
            );
            if ( $redirect ) {
                $result['redirect'] = $redirect;
            }
            echo json_encode( $result );
            die();
        }
        
        /**
         * Add new profit page
         *
         * @return int
         */
        protected function _add_profit()
        {
            global $user_ID;
            
            if ( empty( $_POST['name'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['name'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            $dataPost = array(
                'post_status'  => $status, 
                'post_type'    => 'ipprof-item', 
                'post_author'  => $user_ID,
                'post_title'   => $title,
                'post_content' => ''
            );
            if ( ! empty( $_POST['slug'] )) {
                $dataPost['post_name'] = apply_filters( 'pre_term_name', $_POST['slug'] );
            }
            $result = wp_insert_post( $dataPost );
            return $result;
        }
        
        /**
         * Update profit page
         *
         * @param  object $profit
         * @return boolean
         */
        protected function _update_profit( $profit )
        {
            $result   = 0;
            $dataPost = array();
            if ( ! empty( $_POST['name'] ) && $_POST['name'] != $profit->post_title ) {
                $dataPost['post_title'] = apply_filters( 'single_post_title', $_POST['name'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            if ( $status != $profit->post_status ) {
                $dataPost['post_status'] = $status;
            }
            if ( ! empty( $dataPost )) {
                $dataPost['ID'] = $profit->ID;
                $result = wp_update_post( $dataPost );
            }
            return $result;
        }
        
        /**
         * Processing ajax for delete object
         *
         * @return void
         */
        public function delete()
        {
            $item = false;
            if ( ! empty( $_REQUEST['id'] )) {
                $id   = absint( $_REQUEST['id'] );
                $item = get_post( $id );
            }
            if ( ! $item ) {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Error system', 'wp_profit_page_creator' )
                ) );
                die();
            }
            $result = wp_delete_post( $item->ID, true );
            if ( $result ) {
                echo json_encode( array(
                    'status'   => 1,
                    'message'  => __( 'Successfully remove the profit page', 'wp_profit_page_creator' ),
                    'redirect' => admin_url() . 'admin.php?page=ipprof&delete=1'
                ) );
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to remove the profit page', 'wp_profit_page_creator' )
                ) );
            }
            die();
        }
        
        /**
         * Update offer
         *
         * @return void
         */
        public function offer_update()
        {
            $profitId = 0;
            if ( ! empty( $_POST['profit'] )) {
                $profitId = absint( $_POST['profit'] );
            }
            if ( ! $profit = @get_post( $profitId )) {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed', 'wp_profit_page_creator' )
                ) );
                die();
            
            }
            $redirect = false;
            $is_add   = false;
            $error    = true;
            $ohtml    = false;
            if ( ! empty( $_POST['isadd'] )) {
                $is_add = true;
            }
            if ( ! empty( $_POST['offer_id'] )) {
                $offerId = absint( $_POST['offer_id'] );
                if ( $offer = get_post( $offerId ) ) {
                    $this->_update_offer( $offer );
                } else {
                    $offerId = 0;
                }
            } else {
                $offerId = $this->_add_offer( $profit );
            }
            if ( $offerId )
            {
                $error = false;
                $model = new IPProf_Model();
                $model->save_offer_setting( $offerId );
            }
            if ( $error ) {
                $status   = 0;
                $offerId  = 0;
                if ( $is_add ) {
                    $message = __( 'Failed to create a offer item.', 'wp_profit_page_creator' );
                } else {
                    $message = __( 'Failed to save offer item.', 'wp_profit_page_creator' );
                }
            } else {
                $status = 1;
                if ( $is_add ) 
                {
                    $message = __( 'Successfully create a offer item.', 'wp_profit_page_creator' );
                    $offer   = get_post( $offerId );
                    $list    = new IPProf_List_Offer( $profitId );
                    $ohtml   = $list->create_item( $offer );
                } 
                else {
                    $message = __( 'Successfully to save offer item.', 'wp_profit_page_creator' );
                }
            }
            $result = array(
                'status'   => $status,
                'message'  => $message,
                'offerid'  => $offerId
            );
            if ( $ohtml ) {
                $result['offer'] = $ohtml;
            }
            if ( ! empty( $offerId )) {
                $offer = get_post( $offerId );
                $result['title']  = $offer->post_title;
                if ( $offer->post_status == 'publish' ) {
                    $label = '<span class="ipbe-label ipbe-label-success">' . __( 'Publish' ) . '</span>';
                } else {
                    $label = '<span class="ipbe-label ipbe-label-danger">' . __( 'Draft' ) . '</span>';
                }
                $result['label'] = $label;
            }
            echo json_encode( $result );
            die();
        }
        
        /**
         * Add new offer item
         *
         * @param  object $profit
         * @return int
         */
        protected function _add_offer( $profit )
        {
            global $user_ID;
            
            if ( empty( $_POST['offer_headline'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['offer_headline'] );
            }
            if ( $_POST['offer_status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            $dataPost = array(
                'post_status'  => $status, 
                'post_type'    => 'ipprof-offer-item', 
                'post_author'  => $user_ID,
                'post_title'   => $title,
                'post_content' => '',
                'post_parent'  => $profit->ID
            );
            $result = wp_insert_post( $dataPost );
            return $result;
        }
        
        /**
         * Update offer item
         *
         * @param  object $offer
         * @return boolean
         */
        protected function _update_offer( $offer )
        {
            $result    = 0;
            $data_post = array();
            if ( ! empty( $_POST['offer_headline'] ) && $_POST['offer_headline'] != $offer->post_title ) {
                $data_post['post_title'] = apply_filters( 'single_post_title', $_POST['offer_headline'] );
            }
            if ( $_POST['offer_status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            if ( $status != $offer->post_status ) {
                $data_post['post_status'] = $status;
            }
            if ( ! empty( $data_post )) {
                $data_post['ID'] = $offer->ID;
                $result = wp_update_post( $data_post );
            }
            return $result;
        }
        
        /**
         * Processing ajax for delete offer item
         *
         * @return void
         */
        public function offer_delete()
        {
            $item = false;
            if ( ! empty( $_REQUEST['id'] )) {
                $id   = absint( $_REQUEST['id'] );
                $item = get_post( $id );
            }
            if ( ! $item ) {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Error system', 'wp_profit_page_creator' )
                ) );
                die();
            }
            $result = wp_delete_post( $item->ID, true );
            if ( $result ) {
                echo json_encode( array(
                    'status'   => 1,
                    'message'  => __( 'Successfully remove the offer item', 'wp_profit_page_creator' )
                ) );
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to remove the offer item', 'wp_profit_page_creator' )
                ) );
            }
            die();
        }
        
        /**
         * Get form edit offer
         *
         * @return void
         */
        public function offer_get_edit()
        {
            if ( ! empty( $_GET['id'] )) {
                $offerId = absint( $_GET['id'] );
                if ( $offer = get_post( $offerId ))
                {
                    $admin   = new IPProf_Admin();
                    $admin->offer_form( $offer );
                    die();
                }
            }
            echo '';
            die();
        }
        
        /**
         * Sorting offer item
         *
         * @return void
         */
        public function offer_sort()
        {
            if ( ! empty( $_POST['profit'] ) && ! empty( $_POST['list'] )) 
            {
                $profitId = absint( $_POST['profit'] );
                if ( $profit = get_post( $profitId )) 
                {
                    $_list = trim( $_POST['list'], ',' );
                    $lists = explode( ',', $_list );
                    $index = 1;
                    foreach( $lists as $id )
                    {
                        $id = absint( $id );
                        if ( $offer = get_post( $id )) {
                            $data_post = array(
                                'ID'            => $offer->ID,
                                'menu_order'    => $index
                            );
                            $result = wp_update_post( $data_post );
                        }
                        $index++;
                    }
                }
            }
            echo json_encode( array(
                'status'  => 1
            ));
            die();
        }
        
        /**
         * Get slug
         *
         * @return void
         */
        public function get_slug()
        {
            $data   = array_merge( $_POST, $_GET );
            $postId = 0;
            if ( ! empty( $data['pageid'] )) {
                $postId = absint( $data['pageid'] );
            }
            if ( ! empty( $data['name'] )) {
                $slug = sanitize_title( $data['name'] );
                $slug = wp_unique_post_slug( $slug, $postId, 1, 'ipprof-item', 0 );
                echo json_encode( array(
                    'status' => 1,
                    'slug'   => $slug
                ) );
            } else {
            
                echo json_encode( array(
                    'status' => 0
                ) );
            }
            die();
        }
        
        /**
         * Processing ajax for save setting
         *
         * @return void
         */
        public function setting()
        {
            $error = false;
            if ( ! empty( $_POST['base_path'] )) {
                $path = sanitize_title( $_POST['base_path'] );
                update_option( '__ipprof_base_path', $path );
            }
            if ( $error ) {
                $status = 0;
                $msg    = __( 'Failed to update settings.', 'wp_profit_page_creator' );
            } else {
                $status = 1;
                $msg    = __( 'Successfully to update settings.', 'wp_profit_page_creator' );
            }
            echo json_encode( array(
                'status'    => $status,
                'message'   => $msg
            ) );
            die();
        }
        
        /**
         * Processing ajax for save statistic data
         *
         * @return void
         */
        public function save_stat()
        {
            $error = true;
            if ( ! empty( $_POST['page_id'] ) && ! empty( $_POST['type'] )) 
            {
                $page_id = absint( $_POST['page_id'] );
                if ( $page = get_post( $page_id ))
                {
                    $type  = apply_filters( 'pre_term_name', $_POST['type'] );
                    $model = new IPProf_Model();
                    $model->save_stat( $page, $type );
                    
                    $error = false;
                }
            }
            if ( $error ) {
                $status = 0;
                $msg    = __( 'Failed to save statistic data.', 'wp_profit_page_creator' );
            } else {
                $status = 1;
                $msg    = __( 'Successfully to save statistic data.', 'wp_profit_page_creator' );
            }
            echo json_encode( array(
                'status'    => $status,
                'message'   => $msg
            ) );
            die();
        }
        
        /**
         * Get chart
         *
         * @return void
         */
        public function get_chart()
        {
            $data = array_merge( $_POST, $_GET );
            if ( ! empty( $data['pageid'] ) && ! empty( $data['type'] )) 
            {
                $pageid    = absint( $data['pageid'] );
                $type      = apply_filters( 'pre_term_name', $data['type'] );
                $dateStart = false;
                $dateEnd   = false;
                if ( ! empty( $data['start'] ) && ! empty( $data['end'] )) {
                    $timeEnd   = strtotime( $data['end'] );
                    $timeStart = strtotime( $data['start'] );
                    $dateStart = gmdate("Y-m-d", $timeStart);
                    $dateEnd   = gmdate("Y-m-d", $timeEnd);
                }
                $model     = new IPProf_Model();
                $chart     = array();
                $results   = $model->get_statistic_data( $pageid, $type, $dateStart, $dateEnd );
                
                if ( ! empty( $results )) {
                    foreach( $results as $data ) {
                        $timestamp = strtotime( $data->gdate );
                        $phpdate   = getdate($timestamp);
                        if ( $type == 'ctr' ) {
                            $value = 0;
                            if ( ! empty( $data->click ) && ! empty( $data->view )) {
                                $rate  = 100 * intval($data->click) / intval($data->view);
                                $rate  = round( $rate, 2 );
                                $value = $rate;
                            }
                        } else {
                            $value = (int)$data->value;
                        }
                        $chart[] = array(
                            'value'    => $value,
                            'year'     => $phpdate['year'],
                            'mon'      => $phpdate['mon'] - 1,
                            'day'      => $phpdate['mday'],
                        );
                    }
                }
                if ( $type == 'view' ) {
                    $label = __( 'Views', 'wp_profit_page_creator' );
                } elseif ( $type == 'click' ) {
                    $label = __( 'Clicks', 'wp_profit_page_creator' );
                } elseif ( $type == 'ctr' ) {
                    $label = __( 'CTR (in %)', 'wp_profit_page_creator' );
                } else {
                    $label = __( 'Data', 'wp_profit_page_creator' );
                }
                echo json_encode( array(
                    'status'            => 1,
                    'labelDate'         => __( 'Date', 'wp_profit_page_creator' ),
                    'labelValue'        => $label,
                    'data'              => $chart
                ) );
            } else {
            
                echo json_encode( array(
                    'status' => 0
                ) );
            }
            die();
        }
        
        /**
         * Get more content
         *
         * @return void
         */
        public function get_more()
        {
            $page = false;
            if ( ! empty( $_POST['page_id'] )) {
                $page_id = absint( $_POST['page_id'] );
                $page    = get_post( $page_id );
            }
            if ( ! $page ) {
                echo '';
                die();
            }
            $paged = 1;
            if ( ! empty( $_POST['paged'] )) {
                $paged = absint( $_POST['paged'] );
            }
            $factory = new IPProf_Factory();
            $factory->get_more_offers( $page, $paged );
            die();
        }
    }
}